package com.framsticks.gui;

import com.framsticks.gui.controls.Control;
import com.framsticks.gui.controls.ControlOwner;
import com.framsticks.gui.controls.ValueControl;
import com.framsticks.gui.controls.ValueControlListener;
import com.framsticks.params.AccessInterface;
import com.framsticks.params.Param;
import com.framsticks.params.ValueParam;

import org.apache.log4j.Logger;

import javax.swing.*;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import static com.framsticks.util.lang.Containers.filterInstanceof;

import com.framsticks.util.FramsticksException;
import com.framsticks.util.dispatching.RunAt;

@SuppressWarnings("serial")
public class ObjectPanel extends ModifiablePanel implements ControlOwner {

	private static final Logger log = Logger.getLogger(ObjectPanel.class.getName());

	final protected Map<Param, Control> components = new HashMap<Param, Control>();
	final protected Map<ValueParam, ValueControl> valueControls = new HashMap<ValueParam, ValueControl>();

	public ObjectPanel(Panel.Parameters parameters, Collection<Param> params) {
		super(parameters);

		Gui.fillWithControls(this, params, components, Control.class);
		setName(framsClass.getId());

		for (final ValueControl c : filterInstanceof(components.values(), ValueControl.class)) {
			valueControls.put(c.getParam(), c);
			c.setListener(new ValueControlListener() {
				@Override
				public boolean onChange(Object newValue) {
					if (currentTreeNode == null) {
						return true;
					}
					boolean result = currentTreeNode.changeValue(c, newValue);
					refreshControlButtons();
					return result;
				}
			});
		}

		contentPanel.add(Box.createVerticalGlue());
		this.revalidate();
	}

	@Override
	protected void applyChanges() {
		assert frame.isActive();
		assert currentTreeNode != null;
		currentTreeNode.pushLocalChanges();
	}

	protected void refreshControlButtons() {
		assert frame.isActive();
		applyButton.setEnabled(currentTreeNode.localChanges != null);
	}

	protected static void fillControlsWithValues(Map<ValueControl, Object> map) {
	}

	@Override
	public void pullValuesFromLocalToUser(AccessInterface access) {
		assert currentTreeNode != null;
		assert currentTreeNode.path.getTree().isActive();
		log.debug("refreshing components");

		final Map<ValueControl, Object> values = new HashMap<ValueControl, Object>();
		for (Map.Entry<ValueParam, ValueControl> e : valueControls.entrySet()) {
			values.put(e.getValue(), access.get(e.getKey().getId(), Object.class));
		}

		frame.dispatch(new RunAt<Frame>(frame) {
			@Override
			protected void runAt() {
				if (currentTreeNode.localChanges != null) {
					for (Map.Entry<ValueControl, Object> e : currentTreeNode.localChanges.entrySet()) {
						values.put(e.getKey(), e.getValue());
					}
				}
				for (Map.Entry<ValueControl, Object> e : values.entrySet()) {
					e.getKey().pushValueToUserInterface(e.getValue());
				}
				refreshControlButtons();
				ObjectPanel.this.revalidate();
			}
		});

	}

	@Override
	public String getTitle() {
		return "Properties";
	}

	@Override
	public JPanel getPanel() {
		return contentPanel;
	}

	@Override
	public TreeNode getCurrentTreeNode() {
		return super.getCurrentTreeNode();
	}

	@Override
	public void handle(FramsticksException exception) {
		frame.handle(exception);
	}

	// public void updateValue() {
	//	//assert panel.getFrame().isActive();

	//	final Node n = panel.getCurrentNode();
	//	panel.getBrowser().getManager().invokeLater(new Runnable() {
	//		@Override
	//		public void run() {
	//			Object v = n.getAccess().get(param, Object.class);
	//			if (v == null) {
	//				v = param.getDef(Object.class);
	//			}
	//			final Object fv = v;
	//			panel.getBrowser().invokeLater(new Runnable() {
	//				@Override
	//				public void run() {
	//					setValueImpl(fv);
	//				}
	//			});
	//		}
	//	});
	// }

}
