package com.framsticks.gui.console;

import java.util.LinkedList;
import java.util.List;


import com.framsticks.communication.ClientSideManagedConnection;
import com.framsticks.communication.ClientSideResponseFuture;
import com.framsticks.communication.File;
import com.framsticks.communication.Request;
import com.framsticks.communication.Response;
import com.framsticks.communication.queries.ApplicationRequest;
import com.framsticks.core.Path;
import static com.framsticks.core.TreeOperations.*;
import com.framsticks.gui.SwingDispatcher;
import com.framsticks.params.CompositeParam;
import com.framsticks.params.annotations.AutoAppendAnnotation;
import com.framsticks.params.annotations.FramsClassAnnotation;
import com.framsticks.remote.RemoteTree;
import com.framsticks.util.FramsticksException;
import com.framsticks.util.dispatching.FutureHandler;
import com.framsticks.util.lang.Casting;
import com.framsticks.util.lang.Containers;
import com.framsticks.util.lang.Pair;
import com.framsticks.util.lang.Strings;

@FramsClassAnnotation
public class ManagedConsole extends InteractiveConsole {

	protected RemoteTree tree;

	/**
	 * @param connection
	 */
	public ManagedConsole() {
	}

	/**
	 * @return the connection
	 */
	@Override
	public ClientSideManagedConnection getConnection() {
		return (ClientSideManagedConnection) connection;
	}

	protected void sendImplementation(String line) {
		if (!connection.isConnected()) {
			throw new FramsticksException().msg("not connected").arg("console", this);
		}
		//Move that to managed connection
		ApplicationRequest request;
		try {
			Pair<String, String> command = Strings.splitIntoPair(line, ' ', "");
			request = Casting.throwCast(ApplicationRequest.class, Request.createRequestByTypeString(command.first));
			request.parseRest(command.second);
		} catch (FramsticksException e) {
			throw new FramsticksException().msg("invalid line").arg("line", line).cause(e);
		}

		paintLine(line);

		getConnection().send(request, SwingDispatcher.getInstance(), new ClientSideResponseFuture(this) {
			@Override
			protected void processOk(Response response) {
				for (File f : response.getFiles()) {
					consolePainter.paintMessage(f);
				}
			}
		});
	}

	@Override
	protected void findCompletionPropositions(final String prefix) {
		Pair<CharSequence, CharSequence> command = Request.takeIdentifier(prefix);

		Casting.throwCast(ApplicationRequest.class, Request.createRequestByTypeString(command.first.toString()));
		// Pair<String, String> rest = Strings
		Pair<CharSequence, CharSequence> rest = Request.takeIdentifier(command.second);
		if (rest == null) {
			List<String> propositions = new LinkedList<String>();
			propositions.add(command.first.toString() + " /");
			processCompletionResult(prefix, propositions);
			return;
		}

		final String textual = rest.first.toString();
		if (!textual.startsWith("/")) {
			throw new FramsticksException().msg("invalid line").arg("line", prefix);
		}
		// final Iterator<String> iterator = Path.splitPath(textual);

		resolve(tree, textual, new FutureHandler<Path>(this) {

			@Override
			protected void result(Path path) {
				if (!textual.startsWith(path.getTextual())) {
					throw new FramsticksException().msg("invalid state").arg("line", prefix).arg("path", path);
				}

				List<String> propositions = new LinkedList<String>();
				String remaining = textual.substring(path.getTextual().length());
				remaining = Path.PathBuilder.splitPath(remaining).next();

				for (CompositeParam p : Containers.filterInstanceof(bindAccess(path).getParams(), CompositeParam.class)) {
					if (p.getId().startsWith(remaining)) {
						propositions.add(p.getId());
					}
				}
				processCompletionResult(prefix, propositions);
			}
		});


	}

	@AutoAppendAnnotation
	public void setTree(RemoteTree tree) {
		this.tree = tree;
		connection = tree.getConnection();
	}
}
